pub enum DnsOptDataType {
None,
StrList,
U8List,
U16,
U16List,
U32,
U32List,
InAddr4List,
InAddr6List,
Bin,
Name,
}Expand description
Data type for option records for keys like OPT_OPTIONS and
HTTPS_PARAMS.
Returned by DnsRr::opt_datatype() to indicate the best match for
interpreting an option record value.
Variants
None
No value allowed for this option.
StrList
List of strings, each prefixed with a single octet representing the length.
U8List
List of 8-bit integers, concatenated.
U16
16-bit integer in network byte order.
U16List
List of 16-bit integers in network byte order, concatenated.
U32
32-bit integer in network byte order.
U32List
List of 32-bit integers in network byte order, concatenated.
InAddr4List
List of IPv4 addresses in network byte order, concatenated.
InAddr6List
List of IPv6 addresses in network byte order, concatenated.
Bin
Binary data.
Name
DNS domain name format.