DnsRecord

Struct DnsRecord 

pub struct DnsRecord { /* private fields */ }
An owned DNS record (message), wrapping ares_dns_record_t.

A DnsRecord represents a complete DNS message including header fields, question section, and resource record sections (answer, authority, additional).

Created by parsing wire-format data with DnsRecord::parse().

Implementations

Source

impl DnsRecord

pub fn parse(data: &[u8], flags: DnsParseFlags) -> Result<DnsRecord>

Parse a complete DNS message from wire-format bytes.

flags controls parsing behaviour; use DnsParseFlags::empty() for default parsing.

pub fn id(&self) -> u16

Returns the DNS query ID.

pub fn flags(&self) -> DnsFlags

Returns the DNS record flags.

pub fn opcode(&self) -> DnsOpcode

Returns the DNS record opcode.

pub fn rcode(&self) -> DnsRcode

Returns the DNS record rcode.

pub fn query_count(&self) -> usize

Returns the count of queries in the DNS record.

pub fn query_get(&self, idx: usize) -> Result<(&str, DnsRecordType, DnsCls)>

Get the query at the given index.

Returns (name, record_type, class) on success.

pub fn queries(&self) -> impl Iterator<Item = (&str, DnsRecordType, DnsCls)>

Returns an iterator over the queries in the DNS record.

pub fn rr_count(&self, section: DnsSection) -> usize

Returns the count of resource records in the given section.

pub fn rr(&self, section: DnsSection, idx: usize) -> Option<&DnsRr>

Fetch a non-writable resource record by section and index.

Returns None if the index is out of range.

pub fn rrs(&self, section: DnsSection) -> impl Iterator<Item = &DnsRr>

Returns an iterator over the resource records in the given section.

pub fn new( id: u16, flags: DnsFlags, opcode: DnsOpcode, rcode: DnsRcode, ) -> Result<DnsRecord>

Create a new DNS record object.

When building a query to send via Channel::send_dnsrec(), set id to 0.

Examples
use c_ares::{DnsRecord, DnsFlags, DnsOpcode, DnsRcode, DnsRecordType, DnsCls};

let mut record = DnsRecord::new(
    0,
    DnsFlags::RD,
    DnsOpcode::Query,
    DnsRcode::NoError,
).unwrap();
record.query_add("example.com", DnsRecordType::A, DnsCls::IN).unwrap();
pub fn set_id(&mut self, id: u16) -> &mut Self

Overwrite the DNS query id.

pub fn query_add( &mut self, name: &str, qtype: DnsRecordType, qclass: DnsCls, ) -> Result<&mut Self>

Add a query to the DNS record.

Typically a record will have only one query. Most DNS servers will reject queries with more than one question.

pub fn query_set_name(&mut self, idx: usize, name: &str) -> Result<&mut Self>

Replace the question name at the given index.

This may be used when performing a search with aliases.

pub fn query_set_type( &mut self, idx: usize, qtype: DnsRecordType, ) -> Result<&mut Self>

Replace the question type at the given index.

This may be used when needing to query more than one address class (e.g. A and AAAA).

pub fn rr_add( &mut self, section: DnsSection, name: &str, rr_type: DnsRecordType, rclass: DnsCls, ttl: u32, ) -> Result<&mut DnsRr>

Add a resource record to the DNS record.

Returns a mutable reference to the created resource record for setting RR-specific fields.

pub fn rr_mut(&mut self, section: DnsSection, idx: usize) -> Option<&mut DnsRr>

Fetch a writable resource record by section and index.

Returns None if the index is out of range.

pub fn rrs_mut( &mut self, section: DnsSection, ) -> impl Iterator<Item = &mut DnsRr>

Returns an iterator over mutable resource records in the given section.

pub fn rr_del(&mut self, section: DnsSection, idx: usize) -> Result<&mut Self>

Remove the resource record at the given section and index.

pub fn write(&self) -> Result<AresBuf>

Write a complete DNS message to wire format.

pub fn try_clone(&self) -> Result<DnsRecord>

Create a deep copy of this DNS record.