pub enum DnsRrKey {
Show 58 variants
A_ADDR,
NS_NSDNAME,
CNAME_CNAME,
SOA_MNAME,
SOA_RNAME,
SOA_SERIAL,
SOA_REFRESH,
SOA_RETRY,
SOA_EXPIRE,
SOA_MINIMUM,
PTR_DNAME,
HINFO_CPU,
HINFO_OS,
MX_PREFERENCE,
MX_EXCHANGE,
TXT_DATA,
SIG_TYPE_COVERED,
SIG_ALGORITHM,
SIG_LABELS,
SIG_ORIGINAL_TTL,
SIG_EXPIRATION,
SIG_INCEPTION,
SIG_KEY_TAG,
SIG_SIGNERS_NAME,
SIG_SIGNATURE,
AAAA_ADDR,
SRV_PRIORITY,
SRV_WEIGHT,
SRV_PORT,
SRV_TARGET,
NAPTR_ORDER,
NAPTR_PREFERENCE,
NAPTR_FLAGS,
NAPTR_SERVICES,
NAPTR_REGEXP,
NAPTR_REPLACEMENT,
OPT_UDP_SIZE,
OPT_VERSION,
OPT_FLAGS,
OPT_OPTIONS,
TLSA_CERT_USAGE,
TLSA_SELECTOR,
TLSA_MATCH,
TLSA_DATA,
SVCB_PRIORITY,
SVCB_TARGET,
SVCB_PARAMS,
HTTPS_PRIORITY,
HTTPS_TARGET,
HTTPS_PARAMS,
URI_PRIORITY,
URI_WEIGHT,
URI_TARGET,
CAA_CRITICAL,
CAA_TAG,
CAA_VALUE,
RAW_RR_TYPE,
RAW_RR_DATA,
}Expand description
DNS resource record field keys.
Each variant identifies a specific field within a specific record type.
The key determines which getter/setter method is appropriate (e.g.
get_addr for INADDR fields, get_str for NAME/STR fields).
Variants
A_ADDR
A record: address (INADDR).
NS_NSDNAME
NS record: name server domain name (NAME).
CNAME_CNAME
CNAME record: canonical name (NAME).
SOA_MNAME
SOA record: primary source of data (NAME).
SOA_RNAME
SOA record: responsible mailbox (NAME).
SOA_SERIAL
SOA record: serial number (U32).
SOA_REFRESH
SOA record: refresh interval (U32).
SOA_RETRY
SOA record: retry interval (U32).
SOA_EXPIRE
SOA record: expire limit (U32).
SOA_MINIMUM
SOA record: minimum TTL (U32).
PTR_DNAME
PTR record: pointer domain name (NAME).
HINFO_CPU
HINFO record: CPU (STR).
HINFO_OS
HINFO record: OS (STR).
MX_PREFERENCE
MX record: preference (U16).
MX_EXCHANGE
MX record: exchange domain (NAME).
TXT_DATA
TXT record: data (ABINP).
SIG_TYPE_COVERED
SIG record: type covered (U16).
SIG_ALGORITHM
SIG record: algorithm (U8).
SIG_LABELS
SIG record: labels (U8).
SIG_ORIGINAL_TTL
SIG record: original TTL (U32).
SIG_EXPIRATION
SIG record: signature expiration (U32).
SIG_INCEPTION
SIG record: signature inception (U32).
SIG_KEY_TAG
SIG record: key tag (U16).
SIG_SIGNERS_NAME
SIG record: signer’s name (NAME).
SIG_SIGNATURE
SIG record: signature data (BIN).
AAAA_ADDR
AAAA record: IPv6 address (INADDR6).
SRV_PRIORITY
SRV record: priority (U16).
SRV_WEIGHT
SRV record: weight (U16).
SRV_PORT
SRV record: port (U16).
SRV_TARGET
SRV record: target domain (NAME).
NAPTR_ORDER
NAPTR record: order (U16).
NAPTR_PREFERENCE
NAPTR record: preference (U16).
NAPTR_FLAGS
NAPTR record: flags (STR).
NAPTR_SERVICES
NAPTR record: services (STR).
NAPTR_REGEXP
NAPTR record: regexp (STR).
NAPTR_REPLACEMENT
NAPTR record: replacement (NAME).
OPT_UDP_SIZE
OPT record: UDP size (U16).
OPT_VERSION
OPT record: version (U8).
OPT_FLAGS
OPT record: flags (U16).
OPT_OPTIONS
OPT record: options (OPT).
TLSA_CERT_USAGE
TLSA record: certificate usage (U8).
TLSA_SELECTOR
TLSA record: selector (U8).
TLSA_MATCH
TLSA record: matching type (U8).
TLSA_DATA
TLSA record: certificate association data (BIN).
SVCB_PRIORITY
SVCB record: priority (U16).
SVCB_TARGET
SVCB record: target name (NAME).
SVCB_PARAMS
SVCB record: service parameters (OPT).
HTTPS_PRIORITY
HTTPS record: priority (U16).
HTTPS_TARGET
HTTPS record: target name (NAME).
HTTPS_PARAMS
HTTPS record: service parameters (OPT).
URI_PRIORITY
URI record: priority (U16).
URI_WEIGHT
URI record: weight (U16).
URI_TARGET
URI record: target (NAME).
CAA_CRITICAL
CAA record: critical flag (U8).
CAA_TAG
CAA record: tag/property (STR).
CAA_VALUE
CAA record: value (BINP).
RAW_RR_TYPE
RAW record: RR type (U16).
RAW_RR_DATA
RAW record: RR data (BIN).
Implementations
Sourceimpl DnsRrKey
impl DnsRrKey
Sourcepub fn datatype(self) -> DnsDataType
pub fn datatype(self) -> DnsDataType
Returns the data type associated with this resource record key.
This tells the caller which getter/setter method to use (e.g.
get_addr for DnsDataType::InAddr, get_str for
DnsDataType::Str or DnsDataType::Name).
Sourcepub fn record_type(self) -> DnsRecordType
pub fn record_type(self) -> DnsRecordType
Returns the DNS record type that this key belongs to.
For example, DnsRrKey::A_ADDR returns DnsRecordType::A.